@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim: set ft=python ts=4 sw=4 expandtab:
 
 r"""
@@ -10,7 +9,7 @@ Normalizing a File
 When normalizing a file, the goal is to standardize indenting, comment
 placement, and quoting, without changing the semantics of the file. We do this
 by reading in the file per the spec, and then emitting the same configuration
-in a standard way.  
+in a standard way.
 
 We always emit identifiers unquoted.  We always emit values quoted, using a single
 quote unless the value contains a single quote, in which case we'll use double
@@ -38,7 +37,7 @@ These regular expressions tell us what sort of line we're dealing with:
     - **List line:** ``(^\s*)(list)(\s+)(.*?)(\s*$)``
 
 Any line that does not match one of these regular expressions is an invalid
-line per the specification.  
+line per the specification.
 
 We can simplify these regular expressions into a single check:
 
@@ -46,21 +45,21 @@ We can simplify these regular expressions into a single check:
 
 With this regular expression, if group #4 is `#`, then we have a comment,
 with the comment text in group #5 and leading whitespace in group #3.  Otherwise,
-group #8 gives us the type of the line (``package``, ``config``, ``option`` 
+group #8 gives us the type of the line (``package``, ``config``, ``option``
 or ``list``) and group #10 gives us the remainder of the line after the type.
 
 Next, we need to parse the data on each line according to the individual rules
 for the type of line.  The UCI restrictions on identifiers are enforced,
 including that empty identifiers are not legal.  We also enforce quoting rules,
-including that quotes must match.  
+including that quotes must match.
 
 The spec is silent about embedded and escaped quotes within option values.
 I've chosen to assume that a double-quoted string may contain single quotes and
-vice-versa (like in Python or Perl) but that escaped quotes are not allowed.  
+vice-versa (like in Python or Perl) but that escaped quotes are not allowed.
 
 We do not validate boolean values.  The spec supports a specific list, but you
 can't really identify from looking at the file whether an option is supposed to
-be a boolean or just a string. 
+be a boolean or just a string.
 
 Regardless, if the first regular expression matches a line, and the second
 regular expression does not, then the line isn't valid and we can't process it.
@@ -103,7 +102,7 @@ discussed above.
 UCI Syntax Specification
 ========================
 
-*Note:* This section was taken from the OpenWRT UCI_ documentation.  
+*Note:* This section was taken from the OpenWRT UCI_ documentation.
 
 The UCI configuration files usually consist of one or more config statements,
 so called sections with one or more option statements defining the actual
@@ -116,13 +115,13 @@ are considered a comment and ignored.
 Below is an example of a simple configuration file::
 
     package 'example'
-     
+
     config 'example' 'test'
             option   'string'      'some value'
             option   'boolean'     '1'
             list     'collection'  'first item'
             list     'collection'  'second item'
-        
+
 The config ``'example' 'test'`` statement defines the start of a section with
 the type ``example`` and the name ``test``. There can also be so called anonymous
 sections with only a type, but no name identifier. The type is important for
@@ -171,12 +170,12 @@ accept identifiers that include a dash, regardless of what the spec says.)*
 .. _UCI: https://openwrt.org/docs/guide-user/base-system/uci
 """
 
-from __future__ import annotations  # see: https://stackoverflow.com/a/33533514/2907667
-
 import re
 import typing
 from abc import ABC, abstractmethod
-from typing import List, Optional, Sequence, TextIO, Tuple
+from collections.abc import Sequence
+from pathlib import Path
+from typing import TextIO
 
 # Standard indent of 4 spaces
 _INDENT = "    "
@@ -188,7 +187,6 @@ _LINE_REGEX = re.compile(r"(^\s*$)|((^\s*)(#)(.*$))|((^\s*)(package|config|optio
 _PACKAGE_REGEX = re.compile(r"(^)((([\"'])([a-zA-Z0-9_-]+)(?:\4))|([a-zA-Z0-9_-]+))((\s*)(#.*))?($)")
 
 # Matches the remainder of a config line
-# pylint: disable=line-too-long
 _CONFIG_REGEX = re.compile(
     r"(^)((([\"'])([a-zA-Z0-9_-]+)(?:\4))|([a-zA-Z0-9_-]+))((\s+)((([\"'])([a-zA-Z0-9_-]+)(?:\11))|([a-zA-Z0-9_-]+)))?((\s*)(#.*))?($)"
 )
@@ -205,97 +203,97 @@ def _contains_single(string: str) -> bool:
     return match is not None
 
 
-def _parse_line(lineno: int, line: str) -> Optional[UciLine]:  # pylint: disable=unsubscriptable-object
+def _parse_line(lineno: int, line: str) -> "UciLine | None":
     """Parse a line, raising UciParseError if it is not valid."""
     match = _LINE_REGEX.match(line)
     if not match:
-        raise UciParseError("Error on line %d: unrecognized line type" % lineno)
+        raise UciParseError(f"Error on line {lineno}: unrecognized line type")
     if match[4] == "#":
         return _parse_comment(lineno, match[3], match[5])
-    elif match[8]:
+    if match[8]:
         if match[8] == "package":
             return _parse_package(lineno, match[10])
-        elif match[8] == "config":
+        if match[8] == "config":
             return _parse_config(lineno, match[10])
-        elif match[8] == "option":
+        if match[8] == "option":
             return _parse_option(lineno, match[10])
-        elif match[8] == "list":
+        if match[8] == "list":
             return _parse_list(lineno, match[10])
     return None
 
 
-def _parse_package(lineno: int, remainder: str) -> UciPackageLine:
+def _parse_package(lineno: int, remainder: str) -> "UciPackageLine":
     """Parse a package line, raising UciParseError if it is not valid."""
     match = _PACKAGE_REGEX.match(remainder)
     if not match:
-        raise UciParseError("Error on line %d: invalid package line" % lineno)
-    name = match[5] if match[5] else match[6]
+        raise UciParseError(f"Error on line {lineno}: invalid package line")
+    name = match[5] or match[6]
     comment = match[9]
     return UciPackageLine(name=name, comment=comment)
 
 
-def _parse_config(lineno: int, remainder: str) -> UciConfigLine:
+def _parse_config(lineno: int, remainder: str) -> "UciConfigLine":
     """Parse a config line, raising UciParseError if it is not valid."""
     match = _CONFIG_REGEX.match(remainder)
     if not match:
-        raise UciParseError("Error on line %d: invalid config line" % lineno)
-    section = match[5] if match[5] else match[6]
-    name = match[12] if match[12] else match[9]
+        raise UciParseError(f"Error on line {lineno}: invalid config line")
+    section = match[5] or match[6]
+    name = match[12] or match[9]
     comment = match[16]
     return UciConfigLine(section=section, name=name, comment=comment)
 
 
-def _extract_data_of_remainder_match(match: typing.Match[str]) -> Tuple[str, str, str]:
+def _extract_data_of_remainder_match(match: typing.Match[str]) -> tuple[str, str, str]:
     """Extracts a 3-tuple containing (name,value,comment) out of a {_OPTION_REGEX, LIST_REGEX} matcher"""
-    name = match[5] if match[5] else match[6]
+    name = match[5] or match[6]
     value = ""
     if match[11]:
         value = match[11]
-    elif match[8] not in ('""', "''"):
+    elif match[8] not in {'""', "''"}:
         value = match[8]
     comment = match[15]
     return name, value, comment
 
 
-def _parse_option(lineno: int, remainder: str) -> UciOptionLine:
+def _parse_option(lineno: int, remainder: str) -> "UciOptionLine":
     """Parse an option line, raising UciParseError if it is not valid."""
     match = _OPTION_REGEX.match(remainder)
     if not match:
-        raise UciParseError("Error on line %d: invalid option line" % lineno)
+        raise UciParseError(f"Error on line {lineno}: invalid option line")
     name, value, comment = _extract_data_of_remainder_match(match)
     return UciOptionLine(name=name, value=value, comment=comment)
 
 
-def _parse_list(lineno: int, remainder: str) -> UciListLine:
+def _parse_list(lineno: int, remainder: str) -> "UciListLine":
     """Parse a list line, raising UciParseError if it is not valid."""
     match = LIST_REGEX.match(remainder)
     if not match:
-        raise UciParseError("Error on line %d: invalid list line" % lineno)
+        raise UciParseError(f"Error on line {lineno}: invalid list line")
     name, value, comment = _extract_data_of_remainder_match(match)
     return UciListLine(name=name, value=value, comment=comment)
 
 
-def _parse_comment(_lineno: int, prefix: str, remainder: str) -> UciCommentLine:
+def _parse_comment(_lineno: int, prefix: str, remainder: str) -> "UciCommentLine":
     """Parse a comment-only line, raising UciParseError if it is not valid."""
     indented = len(prefix) > 0 if prefix else False  # all we care about is whether it's indented, not the actual indent
-    comment = "#%s" % remainder
+    comment = f"#{remainder}"
     return UciCommentLine(comment=comment, indented=indented)
 
 
-def _serialize_identifier(prefix: str, identifier: Optional[str]) -> str:  # pylint: disable=unsubscriptable-object
+def _serialize_identifier(prefix: str, identifier: str | None) -> str:
     """Serialize an identifier, which is never quoted."""
-    return "%s%s" % (prefix, identifier) if identifier else ""
+    return f"{prefix}{identifier}" if identifier else ""
 
 
 def _serialize_value(prefix: str, value: str) -> str:
     """Serialize an identifier, which is quoted if it contains whitespace or a quote character."""
     quote = "'" if not _contains_single(value) else '"'
-    return "%s%s%s%s" % (prefix, quote, value, quote)
+    return f"{prefix}{quote}{value}{quote}"
 
 
-def _serialize_comment(prefix: str, comment: Optional[str]) -> str:  # pylint: disable=unsubscriptable-object
+def _serialize_comment(prefix: str, comment: str | None) -> str:
     """Serialize a comment, with an optional prefix."""
-    return "%s%s" % (prefix, comment) if comment else ""
+    return f"{prefix}{comment}" if comment else ""
 
 
 class UciParseError(ValueError):
@@ -317,7 +315,7 @@ class UciLine(ABC):
 class UciPackageLine(UciLine):
     """A package line in a UCI config file."""
 
-    def __init__(self, name: str, comment: Optional[str] = None) -> None:
+    def __init__(self, name: str, comment: str | None = None) -> None:
         self.name = name
         self.comment = comment
 
@@ -325,13 +323,13 @@ class UciPackageLine(UciLine):
         """Serialize the line in normalized form."""
         name_field = _serialize_identifier("package ", self.name)
         comment_field = _serialize_comment("  ", self.comment)
-        return "%s%s\n" % (name_field, comment_field)
+        return f"{name_field}{comment_field}\n"
 
 
 class UciConfigLine(UciLine):
     """A config line in a UCI config file."""
 
-    def __init__(self, section: str, name: Optional[str] = None, comment: Optional[str] = None) -> None:
+    def __init__(self, section: str, name: str | None = None, comment: str | None = None) -> None:
         self.section = section
         self.name = name
         self.comment = comment
@@ -341,13 +339,13 @@ class UciConfigLine(UciLine):
         section_field = _serialize_identifier("\nconfig ", self.section)
         name_field = _serialize_identifier(" ", self.name)
         comment_field = _serialize_comment("  ", self.comment)
-        return "%s%s%s\n" % (section_field, name_field, comment_field)
+        return f"{section_field}{name_field}{comment_field}\n"
 
 
 class UciOptionLine(UciLine):
     """An option line in a UCI config file."""
 
-    def __init__(self, name: str, value: str, comment: Optional[str] = None) -> None:
+    def __init__(self, name: str, value: str, comment: str | None = None) -> None:
         self.name = name
         self.value = value
         self.comment = comment
@@ -357,13 +355,13 @@ class UciOptionLine(UciLine):
         name_field = _serialize_identifier(_INDENT + "option ", self.name)
         value_field = _serialize_value(" ", self.value)
         comment_field = _serialize_comment("  ", self.comment)
-        return "%s%s%s\n" % (name_field, value_field, comment_field)
+        return f"{name_field}{value_field}{comment_field}\n"
 
 
 class UciListLine(UciLine):
     """A list line in a UCI config file."""
 
-    def __init__(self, name: str, value: str, comment: Optional[str] = None) -> None:
+    def __init__(self, name: str, value: str, comment: str | None = None) -> None:
         self.name = name
         self.value = value
         self.comment = comment
@@ -373,13 +371,13 @@ class UciListLine(UciLine):
         name_field = _serialize_identifier(_INDENT + "list ", self.name)
         value_field = _serialize_value(" ", self.value)
         comment_field = _serialize_comment("  ", self.comment)
-        return "%s%s%s\n" % (name_field, value_field, comment_field)
+        return f"{name_field}{value_field}{comment_field}\n"
 
 
 class UciCommentLine(UciLine):
     """A comment line in a UCI config file."""
 
-    def __init__(self, comment: str, indented: bool = False) -> None:
+    def __init__(self, comment: str, *, indented: bool = False) -> None:
         self.comment = comment
         self.indented = indented
 
@@ -387,37 +385,38 @@ class UciCommentLine(UciLine):
         """Serialize the line in normalized form."""
         indent = _INDENT if self.indented else ""
         comment_field = _serialize_comment(indent, self.comment)
-        return "%s\n" % comment_field
+        return f"{comment_field}\n"
 
 
 class UciFile:
-    def __init__(self, lines: List[UciLine]) -> None:
+    def __init__(self, lines: list[UciLine]) -> None:
         self.lines = lines
 
-    def normalized(self) -> List[str]:
+    def normalized(self) -> list[str]:
         """Return a list of normalized lines comprising the file."""
         # We join the lines first and then re-split so we don't end up with lines that have an embedded newline
         return "".join([line.normalized() for line in self.lines]).splitlines(keepends=True)
 
-    # pylint: disable=invalid-name
     @staticmethod
-    def from_file(path: str) -> UciFile:
+    def from_file(path: str | Path) -> "UciFile":
         """Generate a UciFile from a file on disk."""
-        with open(path, "r") as fp:  # pylint: disable=unspecified-encoding
-            return UciFile.from_fp(fp)
+        source = path if isinstance(path, Path) else Path(path)
+        return UciFile.from_text(source.read_text(encoding=None))  # use platform-specific encoding
 
     @staticmethod
-    def from_fp(fp: TextIO) -> UciFile:
+    def from_fp(fp: TextIO) -> "UciFile":
         """Generate a UciFile from the contents of a file pointer."""
         return UciFile.from_lines(fp.readlines())
 
     @staticmethod
-    def from_lines(lines: Sequence[str]) -> UciFile:
+    def from_text(text: str) -> "UciFile":
+        return UciFile.from_lines(text.splitlines(keepends=True))
+
+    @staticmethod
+    def from_lines(lines: Sequence[str]) -> "UciFile":
         """Generate a UciFile from a list of lines."""
-        lineno = 0
-        ucilines: List[UciLine] = []
-        for line in lines:
-            lineno += 1
+        ucilines: list[UciLine] = []
+        for lineno, line in enumerate(lines, start=1):
             parsed = _parse_line(lineno, line)
             if parsed:
                 ucilines.append(parsed)
